local FriendsAlert = CreateFrame("Frame", "FriendsAlert")
local default_settings = {}
FriendsAlert_SavedOptions = {}
local settings = default_settings
local BNGetFriendInfoByID, BNGetFriendInfo, BNGetNumFriends = BNGetFriendInfoByID, BNGetFriendInfo, BNGetNumFriends
local lastAlert = {}

FriendsAlert:RegisterEvent("ADDON_LOADED")

FriendsAlert:SetScript("OnEvent", function(self, event, ...)
	if event == "ADDON_LOADED" and select(1, ...) == "FriendsAlert" then
		FriendsAlert:RegisterEvent("BN_FRIEND_ACCOUNT_ONLINE")
		settings = FriendsAlert_SavedOptions or default_settings
	elseif event == "BN_FRIEND_ACCOUNT_ONLINE" then
		local ID = select(1, ...)
		local _, name, battleTag, _, toonName, _, _, isOnline = BNGetFriendInfoByID(ID)
		--I realize i can just filter isOnline by not registering BN_FRIEND_ACCOUNT_OFFLINE but I plan to do more with mod
		if settings[battleTag] == true then
			if not IsEncounterInProgress() then--Do not do sound alert during raid boss fight
				if lastAlert[battleTag] and GetTime() - lastAlert[battleTag] < 60 then return end--don't alert for same friend more than once every 60 seconds (in case they are switching between toons really fast or something)
				lastAlert[battleTag] = GetTime()
				PlaySoundFile("Sound\\Event Sounds\\OgreEventCheerUnique.ogg", "Master")
			end
		end
--	elseif event == "BN_FRIEND_ACCOUNT_OFFLINE" then
	end
end)

--BNSendFriendInvite(btag/realid, Note)
function FriendsAlert:Print(text)
	_G.DEFAULT_CHAT_FRAME:AddMessage("|cFF33FF99FriendsAlert|r: ".. tostring(text))
end

----------------------
--  Slash Commands  --
----------------------
SLASH_FRIENDSALERT1 = "/friendsalert"
SlashCmdList["FRIENDSALERT"] = function(msg)
	local cmd = msg:lower()
	if cmd:sub(1, 3) == "add" then
		local btag = msg:sub(5)
		if btag then
			settings[btag] = true
			FriendsAlert:Print("|cFF088A08"..btag.."|r has been added to watch.")
		else
			FriendsAlert:Print("|cFF088A08BattleTag|r must be provided (even if it's a realid friend. This mod can only scan battletags since realname is encoded). You can find a friends BTag with /dump BNGetFriendInfo(i) where i is their position in friends list at this moment. It will be 3rd return.")
		end
	elseif cmd:sub(1, 6) == "remove" then
		local btag = msg:sub(8)
		if btag then
			settings[btag] = nil
			FriendsAlert:Print("|cffff0000"..btag.."|r has been removed from watch.")
		else
			FriendsAlert:Print("|cFF088A08BattleTag|r must be provided.")
		end
	elseif cmd:sub(1, 9) == "findbtags" then
		for i=1, select(2, BNGetNumFriends()) do
			local presenceID, name, battleTag = BNGetFriendInfo(i)
			local bTag
			if battleTag and battleTag ~= "" then
				bTag = "|cFF088A08"..battleTag.."|r"
			else
				bTag = "|cffff0000not yet created|r"
			end
			FriendsAlert:Print("|cff0000ff"..name.."'s|r BattleTag is "..bTag)
		end
	elseif cmd:sub(1, 12) == "findallbtags" then
		for i=1, BNGetNumFriends() do
			local presenceID, name, battleTag = BNGetFriendInfo(i)
			local bTag
			if battleTag and battleTag ~= "" then
				bTag = "|cFF088A08"..battleTag.."|r"
			else
				bTag = "|cffff0000not yet created|r"
			end
			FriendsAlert:Print("|cff0000ff"..name.."'s|r BattleTag is "..bTag)
		end
	elseif cmd:sub(1, 4) == "help" then
		FriendsAlert:Print("'/friendsalert add <btag>'. This will add a friend to FriendsAlert watch list. |cffff0000NOTE: This is case sensitive.|r")
		FriendsAlert:Print("'/friendsalert remove <btag>'. This will remove a friend from FriendsAlert watch list. |cffff0000NOTE: This is case sensitive.|r")
		FriendsAlert:Print("'/friendsalert list'. This will list all friends on FriendsAlert watch list.")
		FriendsAlert:Print("'/friendsalert findbtags'. This will help you find BattleTags for all online friends")
		FriendsAlert:Print("'/friendsalert findallbtags'. This will help you find BattleTags for all online & offline friends")
	elseif cmd:sub(1, 4) == "list" then
		FriendsAlert:Print("Alert List")
		FriendsAlert:Print("|cffff0000NOTE: This will fail to list btags that do not match contacts on battle.net friends list.|r")
		for i=1, BNGetNumFriends() do
			local presenceID, name, battleTag = BNGetFriendInfo(i)
			if settings[battleTag] then
				FriendsAlert:Print(battleTag)
			end
		end
	else
		FriendsAlert:Print("Not a valid input. Do '/friendsalert help' for more information.")
	end
end
